@@ -286,6 +286,13 @@
 #define __NR_sys_tee          	77
 #define __NR_sys_vmsplice       75
 #endif
+
+/* Linux syscalls for riscv64 */
+#elif defined(ARCH_RISCV64_H)
+#ifndef __NR_ioprio_set
+#define __NR_ioprio_set		30
+#define __NR_ioprio_get		31
+#endif
 #else
 #warning "Unknown architecture"
 #endif
